'use strict';
require('../css/ht-wrp-reset.scss');
require('../css/ht-wrp.scss');

const hummingtree = (() => {
  const HOST_ID = ht_wrp_host_data.hostId;
  const pluginUrl = ht_wrp_host_data.pluginUrl;
  let urlKey = '';
  // main body
  const hummingtreeMain = document.getElementById('hummingtree');
  const slider = document.querySelector('.hummingtree-slider');
  const toggle = document.querySelector('.hummingtree-toggle');
  const caret = document.querySelector('.hummingtree-caret');
  const head = document.querySelector('.hummingtree-slider-head');
  const body = document.querySelector('.hummingtree-slider-body');
  const footer = document.querySelector('.hummingtree-slider-footer');
  const fLogo = document.querySelector('.hummingtree-footer-logo');
  const button = document.querySelector('.hummingtree-footer-button');
  // message window
  const message = document.querySelector('.hummingtree-message');
  const mLogo = document.querySelector('.hummingtree-message-logo')
  const mShadow = document.querySelector('.hummingtree-message-shadow');
  const mTitle = document.querySelector('.hummingtree-message-title');
  const mHeader = document.querySelector('.hummingtree-message-header');
  const mText = document.querySelector('.hummingtree-message-text');
  const mButton = document.querySelector('.hummingtree-message-button')
  // image preload
  const toggleImg = new Image();
  const adImg = new Image();

  const _createCORSRequest = (method, url) => {
    return new Promise((resolve, reject) => {
      const xhr = new XMLHttpRequest();
      xhr.open(method, url, true);
      xhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
      xhr.onload = () => { xhr.status == 200 ? resolve(xhr.response)
        : reject({ status: xhr.status, res: xhr.response})};
      xhr.onerror = () => { reject(Error('Network Error'))};
      xhr.send();
    });
  };

  const _createWrapper = (data, isDefault) => {
    const { text, link, title, imageUrl, call } = data;
    const adTitle = document.createElement('h3');
    const adText = document.createElement('p');
    const callToAction = document.createElement('p');
    
    callToAction.textContent = call ? `${call}` : 'Learn More';
    adTitle.textContent = `${title}`;
    adText.textContent = `${text}`;
    button.href = `${link}`;
    head.appendChild(adTitle);
    body.appendChild(adText);
    button.appendChild(callToAction);
    toggle.addEventListener('click', _toggleSlider, false);  
    button.addEventListener('click', _conversion, false);

    if (isDefault) {
      hummingtreeMain.classList.add('hummingtree-border');
      caret.src = `${pluginUrl}/hummingtree-wrapper/widget/public/assets/iconExpand.png`;
      slider.classList.remove('hummingtree-slider-inactive');
      slider.classList.add('hummingtree-slider-default', 'hummingtree-slider-closed');
    } else {
      adImg.onload = function() {
        slider.classList.remove('hummingtree-slider-inactive');
        slider.classList.add('hummingtree-slider-closed');
        hummingtreeMain.style.setProperty('background-image', `url(${imageUrl})`, 'important');
        message.classList.add('hummingtree-loaded');
      }
  
      adImg.src = imageUrl;
    }
  };

  const _toggleSlider = () => {
    if (slider.classList.contains('hummingtree-slider-closed')) {
      slider.classList.remove('hummingtree-slider-closed');
      slider.classList.add('hummingtree-slider-open');
      caret.classList.remove('hummingtree-caret-open');
    } else {
      slider.classList.remove('hummingtree-slider-open');
      caret.classList.add('hummingtree-caret-open');
      slider.classList.add('hummingtree-slider-closed');
    }
  };

  const _showError = type => {
    const standardErr = () => {
      mTitle.classList.remove('hummingtree-title-loading');
      mTitle.src = `${pluginUrl}/hummingtree-wrapper/widget/public/assets/ht-text-grey.svg`;
      mLogo.classList.remove('hummingtree-logo-loading');
      mShadow.classList.remove('hummingtree-shadow-loading');
      mShadow.classList.add('hummingtree-shadow-error');
      mHeader.textContent = "Oops!";
      hummingtreeMain.classList.add('hummingtree-border');
    };

    switch (type) {
      case 'invalid id':
        standardErr();

        mLogo.src = `${pluginUrl}/hummingtree-wrapper/widget/public/assets/ht-error.svg`;
        mLogo.classList.add('hummingtree-logo-error');
        mText.textContent = "Your host ID isn't valid. Double check to make sure you've entered all of it.";
        mButton.classList.add('hummingtree-button-active');
        mButton.textContent = "Check Your Settings";
        break;
      case 'missing id':
        standardErr();

        mLogo.src = `${pluginUrl}/hummingtree-wrapper/widget/public/assets/ht-error.svg`;
        mLogo.classList.add('hummingtree-logo-error');
        mText.textContent = "It looks like you haven't entered a host ID.";
        mButton.classList.add('hummingtree-button-active');
        mButton.textContent = "Check Your Settings";
        break;
      case 'blocked':
        standardErr();

        mLogo.src = `${pluginUrl}/hummingtree-wrapper/widget/public/assets/ht-tree-blocked.svg`;
        mLogo.classList.add('hummingtree-logo-blocked');
        mText.classList.add('hummingtree-text-blocked');
        mText.textContent = "It looks like you're blocking our ads. Please consider whitelisting to support this site.";
        break;
      case 'default':
        mTitle.classList.remove('hummingtree-title-loading');
        mTitle.classList.add('hummingtree-title-default');
        mLogo.classList.remove('hummingtree-logo-loading');
        mLogo.src = `${pluginUrl}/hummingtree-wrapper/widget/public/assets/ht-default.svg`;
        mLogo.classList.add('hummingtree-logo-default');
        mShadow.classList.remove('hummingtree-shadow-loading');
        mShadow.classList.add('hummingtree-shadow-error');

        _createWrapper({
          text: 'Make beautiful ads in seconds with HummingTree.',
          link: 'https://hummingtree.co', 
          title: 'Plant Your Ad Here'
        }, true);
        break;
    }

  };

  const _conversion = () => {
    if (urlKey) {
      _createCORSRequest('GET', `https://api.hummingtree.co/delivery/${urlKey}`)
        .catch(err => console.error(err.res));
    }
  };

  const getAd = () => {
    if (!HOST_ID) {
      _showError('missing id');
    } else {
      toggleImg.onload = function() {
        caret.src = toggleImg.src;

        _createCORSRequest('POST', `https://api.hummingtree.co/delivery/${HOST_ID}`)
          .then((data) => {
            const json = JSON.parse(data);
            urlKey = json.urlKey;

            if (json.link === 'https://hummingtree.co' && json.title === 'Plant Your Ad Here') {
              _showError('default');
            } else {
              _createWrapper(json);
            }
          })
          .catch(err => {
            if (err.status === 400) {
              const error = JSON.parse(err.res).error.type;

              if (error === 'malformed id')
                _showError('invalid id');
            } else {
              _showError('default');
            }
            console.error(JSON.parse(err.res));
          });
      };
      toggleImg.onerror = function () {
        _showError('blocked');
      };
      toggleImg.src = 'https://storage.googleapis.com/ads_images_hummingtree/iconExpand.png';
    }
  };

  getAd();
})();